import QUB
from qubopt import MIL
from qubtree import Node
from Buttons import *
from Dialogs import *
from WordReport import *

#def ModelReport(props):
#    config = Node('Properties')
#    config.append( QUB.getModel() )
#    config['tMax'].data = props['tMax']
#    config['tBins'].data = props['tBins']
#    config['Mode'].data = 'model report'
#
#    datasets = QUB.getChosenData()
#    data = [ datasets.find('DataSet') ]
#
#    result = MIL(config, data)
#    if result:
#        result["ResultType"].data = "MdlRpt"
#        QUB.ResultsView.addResults(result)
#    else:
#        QUB.Report('no result?')

#AddTaskButton( 'MdlRpt', # button text
#               ModelReport,   # function
#               'MdlRpt',       # short text for small buttons
#               0,            # show settings dialog on left-click?
#               1,            # show settings dialog on right-click?
#               "Model Report",   # tool-tip
#               "",   # description in "Edit Buttons"
#               "MdlRpt Properties:", # caption for the settings dialog
#               [
#                TextField("tMax", "tMax:", "0.05", acceptFloatGreaterThan(0.0)),
#                TextField("tBins", "tBins:", "64", acceptIntGreaterThan(1))
#                ],
#               'Modeling' )


def NodeLabel(vec):
    if vec.lineComment:
        return '%s (%s)' % (vec.name, vec.lineComment)
    else:
        return vec.name

def WriteMatrix(rep, node):
    mat = node.data
    if mat:
        tbl = [ [NodeLabel(node)] + [x for x in range(1, mat.cols+1)] ]
        for i in range(mat.rows):
            row = mat.row(i)[0:mat.cols]
            tbl.append([i+1] + [x for x in row])
        rep.writeTable(tbl)

def WriteVector(rep, node):
    rep.writeTable( [ [NodeLabel(node)] + [x for x in node.data[:]] ] )

def WriteVectors(rep, *vecs):
    tbl = [ [NodeLabel(vec) for vec in vecs] ]
    n = min([len(vec.data) for vec in vecs])
    for i in range(n):
        tbl.append( [vec.data[i] for vec in vecs] )
    rep.writeTable( tbl )



# initial model image, constraints from WordReport

# modified table includes this data ExpCond
def WriteModelTableWithCond(report, model, expcond):
    tbl = [ ['from', 'to', 'k0', 'k1', 'kEq', 'depend'] ]
    
    rate = model['Rates'].find('Rate')
    while not rate.isNull:
        states = rate['States'].data
        k0     = rate['k0'].data
        dk0    = rate['dk0'].data
        k1     = rate['k1'].data
        dk1    = rate['dk1'].data
        kEq    = k0[1] / k0[0]

        depend = ['', '']
        for i in range(2):
            if rate['P'].data[i]:
                pname = str(rate['PNames'][i].data)
                depend[i] = "%s = %s" % (pname, expcond[pname].data)
            if rate['Q'].data[i]:
                if depend[i]:
                    depend[i] = depend[i] + ', '
                qname = str(rate['QNames'][i].data)
                depend[i] = depend[i] + "%s = %s" % (qname, expcond[qname].data)
        
        tbl.append( [states[0]+1, states[1]+1, k0[0], k1[0], kEq, depend[0]] )
        tbl.append( [states[1]+1, states[0]+1, k0[1], k1[1], kEq, depend[1]] )

        rate = rate.nextSameName()
    report.writeTable(tbl)

def ReportModelTableWithCond(results, context):
    rep = EnsureWordReport(context)
    WriteModelTableWithCond(rep, results.node['Properties']['ModelFile'], results.node['DataSet']['ExpCond'])

Reports.Add('Model Table', ReportModelTableWithCond, ['MdlRpt'])    

def ReportQA(results, context):
    rep = EnsureWordReport(context)
    WriteMatrix(rep, results.node.find("Q"))
    WriteMatrix(rep, results.node.find("A"))

Reports.Add('Q, A', ReportQA, ['MdlRpt'])

def ReportQAstar(results, context):
    rep = EnsureWordReport(context)
    WriteMatrix(rep, results.node.find("Q*"))
    WriteMatrix(rep, results.node.find("A*"))

Reports.Add('Q*, A*', ReportQAstar, ['MdlRpt'])

def ReportModelInfo(results, context):
    rep = EnsureWordReport(context)
    nchan = results.node['Properties']['ModelFile']['ChannelCount'].data[0]
    try:
        nchan = results.node['DataSet']['ExpCond'].find('ChannelCount').data[0]
    except:
        pass
    if nchan == 1:
        rep.write('1 channel')
    else:
        rep.write('%d channels' % nchan)
    rep.write('sampling: %f (ms)' % results.node['DataSet']['sampling'].data[0])
    try:
        rep.write('Tdead: %f (ms)' % results.node['DataSet']['ExpCond']['DeadTime'].data[0])
    except:
        pass
    rep.write('')

Reports.Add('Model Info', ReportModelInfo, ['MdlRpt'])

def ReportP0(results, context):
    rep = EnsureWordReport(context)
    WriteVector(rep, results.node.find('P0'))

Reports.Add('P0', ReportP0, ['MdlRpt'])                

def ReportPeq(results, context):
    rep = EnsureWordReport(context)
    WriteVector(rep, results.node.find('Peq'))

Reports.Add('Peq', ReportPeq, ['MdlRpt'])

def ReportPeqStar(results, context):
    rep = EnsureWordReport(context)
    WriteVector(rep, results.node.find('Peq*'))

Reports.Add('Peq*', ReportPeqStar, ['MdlRpt'])

def ReportQeigen(results, context):
    rep = EnsureWordReport(context)
    WriteVector(rep, results.node.find('Q_eigen'))

Reports.Add('Q eigen', ReportQeigen, ['MdlRpt'])

def ReportQeigenStar(results, context):
    rep = EnsureWordReport(context)
    WriteVector(rep, results.node.find('Q_eigen*'))

Reports.Add('Q eigen*', ReportQeigenStar, ['MdlRpt'])


def WriteTimeConstants(rep, res, suffix):
    meanTC = res.find('MeanTimeConst'+suffix).data
    tc = res.find('TimeConst'+suffix)
    ic = 0

    tbl = [ ['', 'Mean', 'Tau (ms)', 'Amp'] ]
    while not tc.isNull:
        tbl.append( ['class %d'%(ic+1), meanTC[ic], tc.data[0], tc.data[1]] )
        for i in range(2, len(tc.data), 2):
            tbl.append( ['', '', tc.data[i], tc.data[i+1]] )
        tbl.append(['', '', '', ''])
        tc = tc.nextSameName()
        ic = ic + 1

    rep.write('Time Constants'+suffix+':')
    rep.writeTable(tbl[:-1])

def ReportTimeConstants(results, context):
    rep = EnsureWordReport(context)
    WriteTimeConstants(rep, results.node, '')

Reports.Add('Mdl Time Constants', ReportTimeConstants, ['MdlRpt'])

def ReportTimeConstantsStar(results, context):
    rep = EnsureWordReport(context)
    WriteTimeConstants(rep, results.node, '*')

Reports.Add('Mdl Time Constants*', ReportTimeConstantsStar, ['MdlRpt'])


def WriteCurrent(rep, res, suffix):
    Ieq = res['Ieq'+suffix].data[0]
    B = res['I_B'+suffix]
    T = res['I_T'+suffix]
    n = len(B.data)
    
    rep.write('Macroscopic current'+suffix+':')
    eqn = 'I(t) = Ieq'+suffix
    for i in range(1, n+1):
        eqn = eqn + ' + B%s%d*exp(-t/T%s%d)' % (suffix, i, suffix, i)
    rep.write(eqn)

    rep.write('Ieq = %f' % Ieq)
    WriteVectors(rep, B, T)

def ReportCurrent(results, context):
    rep = EnsureWordReport(context)
    WriteCurrent(rep, results.node, '')

Reports.Add('Current', ReportCurrent, ['MdlRpt'])

def ReportCurrentStar(results, context):
    rep = EnsureWordReport(context)
    WriteCurrent(rep, results.node, '*')

Reports.Add('Current*', ReportCurrentStar, ['MdlRpt'])

def WriteFirstLatency(rep, res, suffix):
    Amp = res['FL_Amp'+suffix]
    Tau = res['FL_Tau'+suffix]

    eqn = 'FirstLat(t) ='
    for i in range(len(Amp.data)):
        if i:
            eqn = eqn + ' +'
        eqn = eqn + ' Amp%d*exp(-t/Tau%d)' % (i+1, i+1)
    rep.write(eqn)
    WriteVectors(rep, Amp, Tau)

def ReportFirstLatency(results, context):
    rep = EnsureWordReport(context)
    WriteFirstLatency(rep, results.node, '')

Reports.Add('First Latency', ReportFirstLatency, ['MdlRpt'])    

def ReportFirstLatencyStar(results, context):
    rep = EnsureWordReport(context)
    WriteFirstLatency(rep, results.node, '*')

Reports.Add('First Latency*', ReportFirstLatencyStar, ['MdlRpt'])    

def ReportFirstLatencyPlot(results, context):
    rep = EnsureWordReport(context)
    histgroup = results.node.find('HistogramGroup')
    while not histgroup.isNull:
        if str(histgroup['Histogram']['Title'].data) == 'First Latency':
            rep.write('First Latency:')
            rep.writeHistogram(histgroup['Histogram'])
            return
        histgroup = histgroup.nextSameName()

Reports.Add('First Latency Plot', ReportFirstLatencyPlot, ['MdlRpt'])

def ReportFirstLatencyPlotStar(results, context):
    rep = EnsureWordReport(context)
    histgroup = results.node.find('HistogramGroup')
    while not histgroup.isNull:
        if str(histgroup['Histogram']['Title'].data) == 'First Latency*':
            rep.writeHistogram(histgroup['Histogram'])
            return
        histgroup = histgroup.nextSameName()

Reports.Add('First Latency* Plot', ReportFirstLatencyPlotStar, ['MdlRpt'])

def ReportMeanFirstPassage(results, context):
    rep = EnsureWordReport(context)
    rep.write('Mean first passage i->j [ms]:')
    WriteMatrix(rep, results.node.find("MeanFirstPassage"))
Reports.Add('Mean First Passage', ReportMeanFirstPassage, ['MdlRpt'])
def ReportMeanFirstPassageStar(results, context):
    rep = EnsureWordReport(context)
    rep.write('Mean first passage* i->j [ms]:')
    WriteMatrix(rep, results.node.find("MeanFirstPassage*"))
Reports.Add('Mean First Passage*', ReportMeanFirstPassageStar, ['MdlRpt'])
